local *

{
  :idTweaksOptions
  :idTweaksUtils
} = _G
{ :map } = idTweaksUtils.table
{ :get_db_option, :set_db_option } = idTweaksUtils.db

class Option
  new: (module_instance, key, name, description, values = {}, options = {}, default) =>
    -- print "Option#new:#{key}"

    @order = 0
    @parent = nil
    @type = "group"
    @module = module_instance
    @key = key
    @name = name
    @description = description

    map.each ((v, k) ->
      @["create_#{k}"] = => v
    ), values

    @num_options = 0
    @options = {}
    map.each ((v) ->
      @add_option v
    ), options

    @default = default
    
  get_display_key: =>
    -- print "Option#get_display_key:#{@key}"

    if @parent
      "#{@parent\get_display_key!} #{@key}"
    else
      @key
      
  get_display_value: (value) =>
    if value == true
      "On"
    elseif value == false
      "Off"
    else
      value

  get_full_key: =>
    -- print "Option#get_full_key:#{@key}"

    if @parent and @parent.key ~= @module.name and @parent.key ~= "idTweaks"
      "#{@parent\get_full_key!}.#{@key}"
    else
      @key

  get: =>
    -- print "Option#get:#{@key}"
    
    map.get @get_full_key!, @module.db

  set: (value) =>
    -- print "Option#set_value:#{@key}:#{value}"

    map.set value, @get_full_key!, @module.db
    
    -- if _G.idTweaks\is_bc!
    --   print "#{@get_display_key!}: #{@get_display_value value}"

    @module\check!

  add_value: (key, value) =>
    -- print "Option#add_value:#{@key}:#{key}:#{value}"

    @values[key] = value

  remove_value: (key) =>
    -- print "Option#remove_value:#{@key}:#{key}"

    @values[key] = nil

  get_option: (key) =>
    -- print "Option#get_option:#{@key}"

    @options[key]

  add_option: (option) =>
    -- print "Option#add_option:#{@key}:#{option.key}"

    @options[option.key] = option
    @options[option.key].order = @num_options
    @num_options = @num_options + 1

    option.parent = @

  remove_option: (option) =>
    -- print "Option#remove_option:#{@key}"

    option.parent = nil

    @options[option.key] = nil

  get_aceconfig_defaults: =>
    -- print "Option#get_aceconfig_defaults:#{@key}"

    if @num_options == 0
      @default
    else
      map.map ((v) -> v\get_aceconfig_defaults!), @options

  get_aceconfig_options: =>
    -- print "Option#get_aceconfig_options:#{@key}"

    output =
      order: @order
      type: @type
      name: @name
      desc: @description
      disabled: @create_disabled!
      hidden: @create_hidden!
      get: @create_get!
      set: @create_set!

    if @num_options > 0
      output.args = map.map ((v) -> v\get_aceconfig_options!), @options

    output

  create_disabled: =>
    ->
      -- print "Option#disabled:#{@key}"

      false

  create_hidden: =>
    ->
      -- print "Option#hidden:#{@key}"

      false

  create_get: =>
    ->
      -- print "Option#get:#{@key}"

      @get!

  create_set: =>
    (_, value) ->
      -- print "Option#set:#{@key}"

      @set value

idTweaksOptions.Option = Option
