require "build/utils"

utils = idTweaksUtils

describe 'utils', ->
  describe 'function', ->
    describe 'id', ->
      it 'should return nil for nil', ->
        import id from utils.function
        expected = nil
        actual = id nil
        assert.are.same expected, actual

      it 'should return the value for a value', ->
        import id from utils.function
        expected = 1
        actual = id 1
        assert.are.same expected, actual

    describe 'apply', ->
      it 'should return a function that calls the input function with the arguments', ->
        import apply from utils.function
        applied = apply ((x, y) -> x + y), 5
        expected = 10
        actual = applied 5
        assert.are.same expected, actual

  describe 'string', ->
    describe 'join', ->
      it 'should return the table as a concatenated by sep string', ->
        import join from utils.string
        expected = '1,2,3'
        actual = join ',', {'1', '2', '3'}
        assert.are.same expected, actual

    describe 'split', ->
      it 'should return the concatenated by sep string for the table', ->
        import split from utils.string
        expected = {'1', '2', '3'}
        actual = split ',', '1,2,3'
        assert.are.same expected, actual

        expected = {'accepting', 'accept_hearthstone_bind'}
        actual = split '.', 'accepting.accept_hearthstone_bind'
        assert.are.same expected, actual

  describe 'table', ->
    describe 'list', ->
      describe 'map', ->
        it 'should return a table with fn of v in the input table', ->
          import map from utils.table.list
          expected = {2, 4, 6, 8, 10}
          actual = map ((v) -> v * 2), {1, 2, 3, 4, 5}
          assert.are.same expected, actual

      describe 'filter', ->
        it 'should return a table with values of the input table for which fn holds', ->
          import filter from utils.table.list
          expected = {1, 2, 3, 4, 5}
          actual = filter ((v) -> v <= 5), {1, 2, 3, 4, 5, 6, 7, 8, 9, 10}
          assert.are.same expected, actual

      describe 'reduce', ->
        it 'should return an accumulated result for fn(a,v) in each of t', ->
          import reduce from utils.table.list
          expected = 5
          actual = reduce(((a, v) -> a + v), {1, 1, 1, 1, 1}, 0)
          assert.are.same expected, actual

      describe 'each', ->
        it 'should run fn for each in t', ->
          import each from utils.table.list
          expected = 5
          actual = 0
          each ((v) -> actual = actual + v), {1, 1, 1, 1, 1}
          assert.are.same expected, actual

      describe 'chunk', ->
      describe 'compact', ->

      describe 'concat', ->
        it 'should return a concatenated table of the tables', ->
          import concat from utils.table.list
          expected = {"A", "B", "C", "D", "E", "F"}
          actual = concat {"A", "B", "C"}, {"D", "E", "F"}
          assert.are.same expected, actual

      describe 'difference', ->
      describe 'difference_by', ->
      describe 'difference_with', ->
      describe 'drop', ->
      describe 'drop_right', ->
      describe 'drop_right_while', ->
      describe 'drop_while', ->
      describe 'fill', ->
      describe 'find_index', ->
      describe 'find_last_index', ->
      describe 'flatten', ->
      describe 'flatten_deep', ->
      describe 'flatten_depth', ->
      describe 'from_pairs', ->
      describe 'head', ->
      describe 'index_of', ->
      describe 'initial', ->
      describe 'intersection', ->
      describe 'intersection_by', ->
      describe 'intersection_with', ->
      describe 'join', ->
      describe 'last', ->
      describe 'last_index_of', ->
      describe 'nth', ->
      describe 'pull', ->
      describe 'pull_all', ->
      describe 'pull_by', ->
      describe 'pull_with', ->
      describe 'pull_at', ->
      describe 'remove', ->
      describe 'reverse', ->
      describe 'slice', ->
      describe 'sorted_index', ->
      describe 'sorted_index_by', ->
      describe 'sorted_index_of', ->
      describe 'sorted_last_index', ->
      describe 'sorted_last_index_by', ->
      describe 'sorted_last_index_of', ->
      describe 'sorted_uniq', ->
      describe 'sorted_uniq_by', ->
      describe 'tail', ->
      describe 'take', ->
      describe 'take_right', ->
      describe 'take_right_while', ->
      describe 'take_while', ->
      describe 'union', ->
      describe 'union_by', ->
      describe 'union_with', ->
      describe 'uniq', ->
      describe 'uniq_by', ->
      describe 'uniq_with', ->
      describe 'unzip', ->
      describe 'unzip_with', ->
      describe 'without', ->
      describe 'xor', ->
      describe 'xor_by', ->
      describe 'xor_with', ->
      describe 'zip', ->
      describe 'zip_object', ->
      describe 'zip_object_deep', ->
      describe 'zip_with', ->

    describe 'map', ->
      describe 'get', ->
        it 'should return return the value of the key deep in the table', ->
          import get from utils.table.map
          expected = 3
          actual = get 'a.b.c',
            a:
              b:
                c: 3
          assert.are.same expected, actual

      describe 'set', ->
        it 'should set the value of the key deep in the table', ->
          import set from utils.table.map
          expected = 3
          actual =
            a:
              b:
                c: 1
          set 3, 'a.b.c', actual
          assert.are.same expected, actual.a.b.c

      describe 'map', ->
        it 'should apply the function over each value of the table', ->
          import map from utils.table.map
          expected =
            A: 2
            B: 4
            C: 6
          actual = map ((x) -> x * 2),
            A: 1
            B: 2
            C: 3
          assert.are.same expected, actual

      describe 'reduce', ->
        it 'should return an accumulated result for fn(a,v) in each of t', ->
          import reduce from utils.table.map
          expected = 3
          actual = reduce ((a, v, k) ->
            a = a + v
            a
          ), {
            bob: 1,
            sally: 1,
            tom: 1,
          }, 0
          assert.are.same expected, actual

      describe 'each', ->
        it 'should run fn for each in t', ->
          import each from utils.table.map
          expected = 2
          actual = 0
          each ((v, k) -> actual = actual + v),
            A: 1,
            B: 1,
          assert.are.same expected, actual


      describe 'clone', ->
        it 'should apply the function over each value of the table', ->
          import clone from utils.table.map
          expected =
            A: 1
            B: 2
            C: 3
          actual = clone
            A: 1
            B: 2
            C: 3
          assert.true expected ~= actual
          assert.are.same expected, actual

      describe 'clone_deep', ->
        it 'should apply the function over each value of the table', ->
          import clone_deep from utils.table.map
          expected =
            A: 1
            B: 2
            C: 3
            D:
              E: 4
              F: 5
          actual = clone_deep
            A: 1
            B: 2
            C: 3
            D:
              E: 4
              F: 5
          assert.true expected ~= actual
          assert.are.same expected, actual

      describe 'reverse', ->
        it 'should reverse the keys and values of the table', ->
          import reverse from utils.table.map
          expected =
            A: 'D'
            B: 'E'
            C: 'F'
          actual = reverse
            D: 'A'
            E: 'B'
            F: 'C'
          assert.true expected ~= actual
          assert.are.same expected, actual

      describe 'merge', ->
        it 'should merge the tables', ->
          import merge from utils.table.map
          expected =
            A: 'D'
            B: 'E'
            C: 'F'
            X: 'G'
            Y: 'H'
            Z: 'I'

          actual = merge {
            A: 'D'
            B: 'E'
            C: 'F'
          }, {
            X: 'G'
            Y: 'H'
            Z: 'I'
          }

          assert.true expected ~= actual
          assert.are.same expected, actual
